# Copyright(c) 2000-2011 Association of Universities for Research in Astronomy, Inc.

procedure gemhedit (image, field, value, comment)

# Edit FITS header keyword
# Should be safe and never return status = 1 unless image does not exist
#
# Version  May 31, 2001   IJ, v1.2 release
#          Mar 03, 2002   IJ, v1.3 release
#          Sep 20, 2002   IJ, v1.4 release
#          Feb 03, 2003   IJ  fix segmentation violation bug under Linux7.3
#          Aug 11, 2003   KL, Added 'addonly-' parameter to hedit calls
#          Jan 09, 2009   JH, Propagate comment update for existing keyword
#          Feb 03, 2010   EH, Remove original comment before writing new one
#                             (longer comments remain), add delete option

char    image   {prompt = "Input image"}
char    field   {prompt = "Keyword name"}
char    value   {prompt = "Value"}
char    comment {prompt = "Comment to add for keyword"}
bool    delete  {no,prompt = "Delete keyword?"}
int     status  {0,prompt = "Exit status"}

begin

    char    l_image, l_field, l_value, l_comment
    bool    l_keyexist, l_delete, newcomment
    
    cache ("keypar", "nhedit")

    l_image = image ; l_field = field ; l_value = value ; 
    l_comment = comment ; l_delete = delete
    # Initialize param
    status = 0
    newcomment = yes
    if (no == imaccess(l_image)) {
        print("ERROR - GEMHEDIT: Image "//l_image//" not found")
        status = 1
        bye
    }

    # Make sure the field name is in upper case. Don't capitalise 
    # keywords beginning with i_ EH
    if (substr(l_field, 1, 2) != "i_")
        print (l_field) | translit ("STDIN","a-z","A-Z",del-,coll-) | scan (l_field)

    # Cut keyword to eight if it is longer
    if (strlen(l_field) > 8)
        l_field = substr(l_field,1,8)

    
    # Check if keyword already exists
    #
    # A keyword can exist *and* have an empty value. The original call to 
    # hselect didn't work properly if a keyword existed but had an empty 
    # value. Previously, in this case, it was assumed that the keyword 
    # did not exist, which meant that the nhedit call with add+ was used. 
    # This, unfortunately, wrote the same keyword to the header for a 
    # second time. Using keypar instead allows you to test whether the 
    # keyword actually exists, rather than whether it has a value. EH
    #hselect(l_image,l_field,yes) | scan(l_keycheck)
    
    l_keyexist = no
    keypar(l_image, l_field, silent+)
    if (keypar.found)
        l_keyexist = yes

    if ((l_keyexist == yes) && (l_comment == "")) {
        newcomment = no
        l_comment = "." # leave original comment in places
    }
    
    # Because nhedit was removed from fitsutil and we do want
    # to get backward compatibility for a little while, the first
    # block checks if fitsutil.nhedit exists and if so use it,
    # if not, the following block checks if imutil.nhedit exists and if
    # so use that one.  The final block just catches error cases where
    # no known nhedit is found.
    #
    # It is a lot of code duplication, but hopefully in a couple release
    # we will be able to remove the backward compatible block to fitsutil.
    # When that day comes it will be easy to remove it all in one shot.
    # -- KL,  13 Jan 2010
    
    if ( deftask("fitsutil.nhedit") ) {
        # Delete keyword only if it exists
        if (l_delete) {
            if (l_keyexist == yes) {
                fitsutil.nhedit (l_image, l_field, ".", comment=".", comfile="NULL", \
                    after="", before="", update+, add-, addonly-, delete+, \
                    verify-, show-)
            }

        } else if (l_keyexist == yes) {
            if (newcomment == yes) {
                # Empty the comment field before adding a new comment
                fitsutil.nhedit(l_image, l_field, l_value, comment="", comfile="NULL", \
                    after="", before="", update+, add-, addonly-, delete-, \
                    verify-, show-)
            }
            fitsutil.nhedit(l_image, l_field, l_value, comment=l_comment, comfile="NULL", \
                after="", before="", update+, add-, addonly-, delete-, \
                verify-, show-)

        } else { 
            if (l_value == 'INDEF') {
                fitsutil.nhedit (l_image, l_field, "CHANGEME", comment=l_comment, \
                    comfile="NULL", after="", before="", update+, add+, addonly-, \
                    delete-, verify-, show-)
                fitsutil.nhedit(l_image, l_field, l_value, comment=".", comfile="NULL", \
                    after="", before="", update+, add-, addonly-, delete-, \
                    verify-, show-)
            } else {

                fitsutil.nhedit (l_image, l_field, l_value, comment=l_comment, \
                    comfile="NULL", after="", before="", update+, add+, addonly-, \
                    delete-, verify-, show-)
            }
        }
    } else if ( deftask("imutil.nhedit") ) {
        # Delete keyword only if it exists
        if (l_delete) {
            if (l_keyexist == yes) {
                imutil.nhedit (l_image, l_field, ".", comment=".", comfile="", \
                    after="", before="", update+, add-, addonly-, delete+, \
                    verify-, show-)
            }

        } else if (l_keyexist == yes) {
            if (newcomment == yes) {
                # Empty the comment field before adding a new comment
                imutil.nhedit(l_image, l_field, l_value, comment="", comfile="", \
                    after="", before="", update+, add-, addonly-, delete-, \
                    verify-, show-)
            }
            imutil.nhedit(l_image, l_field, l_value, comment=l_comment, comfile="", \
                after="", before="", update+, add-, addonly-, delete-, \
                verify-, show-)

        } else { 
            if (l_value == 'INDEF') {
                imutil.nhedit (l_image, l_field, "CHANGEME", comment=l_comment, \
                    comfile="", after="", before="", update+, add+, addonly-, \
                    delete-, verify-, show-)
                imutil.nhedit(l_image, l_field, l_value, comment=".", comfile="", \
                    after="", before="", update+, add-, addonly-, delete-, \
                    verify-, show-)
            } else {

                imutil.nhedit (l_image, l_field, l_value, comment=l_comment, \
                    comfile="", after="", before="", update+, add+, addonly-, \
                    delete-, verify-, show-)
            }
        }
    } else {
        print ("ERROR: no nhedit found")
        status = 1
    }
    
end
