# Copyright(c) 2002-2009 Association of Universities for Research in Astronomy, Inc.
#
# GMOS example reductions script:  Typical reduction of longslit observation
#
# This data processing was done to aid the data quality assessment performed
# by the Gemini staff.  The data processing is not designed to give the best
# possible result.  Better signal-to-noise and better cleaning for cosmic-ray
# hits and bad pixels will most likely be possible.  The user of these data
# is encouraged to use the provided co-added images only as guide lines and
# to re-reduce the data to obtain the best possible reduction.
#
# Gemini GMOS data reduction script
# Observation UT date: 2001nov13
# Processor: Kathy Roth, Inger Jorgensen
# Date reduction date: 2002feb27
#
# Brief data description: Longslit data from the SV program GN-2001B-SV-60
# For this test only reducing a subset of the data
#
# Define various directories containing raw data and calibration data
set rawdir=/net/sabrina/staging2/gmos/2001nov13/
set gcalib=../../2001nov22/Basecalib/

# set up the logfile for this reduction
gmos.logfile="GN-2001B-SV-60_log.txt"

# The bias image has been previously made with GBIAS

# Make the flat field -- bias has been overscan corrected
gsflat N20011112S063 N20011112S063_flat \
order=23 rawpath="rawdir$" bias="gcalib$N20011109S136_bias" fl_over+

# Reduce the observations of the science target, the name of the output
# image is defined by the default outpref="gs"
#
# Steps in GSREDUCE:
#    subtract off the bias
#    mosaic the 3 detectors
#    interpolate across the chip gaps for science data
#    flat field correction
#    run GSAPPWAVE to get an approximate wavelength calibration in the 
#       header of the output image
#
gsreduce N20011112S065 rawpath="rawdir$" bias="gcalib$N20011109S136_bias" \
flat="N20011112S063_flat" fl_over+

# Reduce the CuAr B600/500.  The CuAr is not flat fielded, 
# gaps are not fixpixed.
gsreduce N20011112S064 rawpath="rawdir$" fl_flat- \
bias="gcalib$N20011109S136_bias" fl_fixpix- fl_over+

# For CuAr taken through the 2.0 arcsec slit must smooth the data so that
# accurate line centers can be found by GSWAVELENGTH.  Also need to adjust
# the line width, centering radius and minimum separation parameters.

# Establish the wavelength calibration
gswavelength gsN20011112S064 

# Transform the CuAr spectrum, for checking that the transformation is OK.
# Output image name is defined by the default outpref="t"
gstransform gsN20011112S064 wavtran=gsN20011112S064

# Transform the science spectrum
gstransform gsN20011112S065 wavtran=gsN20011112S064

# Sky subtraction, the sample is manually selected after inspection of
# the image
gsskysub tgsN20011112S065 long_sample="550:650,1650:1750" fl_inter+

# Inspect the sky subtracted spectrum
imexa stgsN20011112S065[sci,1]

# Extract the spectrum
gsextract stgsN20011112S065 apwidth=5

# Inspect the extracted spectrum
splot estgsN20011112S065[sci,1]

# The spectrum may also be flux calibrated using GSCALIBRATE.
# This requires that a spectrophometric standard star has been reduced
# and the flux calibration established as outlined in 
#     gmosexamples spec-std
